/*******************************************************************************
	Demonstration Plate Driver Class implementation

	THE SOFTWARE IS PROVIDED \"AS-IS\" AND WITHOUT WARRANTY OF ANY KIND,
	EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
	WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

	IN NO EVENT SHALL PETER KOLLER BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
	INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
	RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
	THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
	OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

	(c) COPYRIGHT 2022 PETER KOLLER.
*******************************************************************************/
#define		WIN32_LEAN_AND_MEAN
#define		DllExport   __declspec(dllexport)
#define		_WIN32_IE		0x600		//must be at least common controls & shell v6.00 (was 500 in API V3.20)
#define		WINVER			0x600
#define		_WIN32_WINNT	0x600
#define		M_ALLOW_ANIMATION			//allows the use of the Windows Animate function
#define		M_UNRESTRICTED_MEMSIZE		//allows the use of memory above 2^32 bytes

#include    <windows.h>
#include    <tchar.h>
#include    <stdlib.h>
#include    <string.h>
#include    <stdio.h>
#include    "RasterOut.h"
#include    "PlateDriver.h"
#include    "DemoPlate.h"
#include    "PlateData.h"

inline BOOL    _flagtest(ULONG  Flags, ULONG Key)
{
	return((BOOL)(Key == (Flags & Key)));
}

const LPPLATEDATA DemoplateDriverClass::EnumPlateData(int &index)
{
	int cnt;
	LPPLATEDATA result;
	for (cnt = 0; pd[cnt].strcIdl.cb != 0; cnt++)
	{
		if (index == cnt)
			result = &pd[cnt];
	}
	index = cnt;
	return result;
}

HANDLE DemoplateDriverClass::Open(LPMETACONTROL pXmlMetadata, PRENDERDATA lpRd, int PlateIndex)
{
	return HANDLE(this);
}

HANDLE DemoplateDriverClass::OpenRasterPage(HANDLE plateHandle, LPCTSTR pDataFile, SIZE plateOffset, SIZE mapSize, int lpiResolution, int dotmapSize)
{
	LPPAGEINFORMATION pgInfo = new PAGEINFORMATION;
	pgInfo->lpiResolution = lpiResolution;
	pgInfo->dotmapSize = dotmapSize;
	pgInfo->xOffset = TranslateCoordinates(HANDLE(pgInfo), plateOffset.cx, false);
	pgInfo->yOffset = TranslateCoordinates(HANDLE(pgInfo), plateOffset.cy, false);
	pgInfo->curOffs = 0;
	return HANDLE(pgInfo);
}

LONG DemoplateDriverClass::TranslateCoordinates(HANDLE handle, LONG value, bool bGetMeasurement)
{
	LPPAGEINFORMATION pgInfo = LPPAGEINFORMATION(handle);
	INT64 result = value;
	if (bGetMeasurement)
	{
		result *= 2540L;
		result /= (pgInfo->lpiResolution * pgInfo->dotmapSize);

	}
	else
	{
		result *= (pgInfo->lpiResolution * pgInfo->dotmapSize);
		result /= 2540L;
	}
	return LONG(result);
}

bool DemoplateDriverClass::WriteRasterData(HANDLE handle, LONG yRow, LPCVOID lpBuffer, size_t rowSize)
{
	//write out the line of data

	return true; //if successful...
}

void DemoplateDriverClass::CloseRasterPage(HANDLE handle)
{
	if (handle)
	{
		delete LPPAGEINFORMATION(handle);
	}
}

void DemoplateDriverClass::Close(HANDLE handle)
{
	//we do not need to delete our own instance
}


UINT64 DemoplateDriverClass::dpiToPixels(LONG offset, int lpiResolution, int dotmapSize)
{
	UINT64 result = offset;
	result *= (lpiResolution * dotmapSize);
	result /= 2540L;
	return result;
}

int DemoplateDriverClass::SetPosition(HANDLE plateHandle, LPPOINT newPosition, ULONG positionFlags)
{
	if (_flagtest(positionFlags, SETPOSN_ENDOFLINE))
	{
		//move to next line
	}
	else if (_flagtest(positionFlags, SETPOSN_MOVETO))
	{
		//move if you can
	}

	return SETPOSN_SUCCESS; //if successful...
}
